package com.css.simulation.resource.scheduler.util;

import api.common.pojo.constants.DictConstants;
import api.common.util.StringUtil;
import com.css.simulation.resource.scheduler.configuration.kubernetes.KubernetesConfiguration;
import com.css.simulation.resource.scheduler.mapper.AutoSubProjectMapper;
import com.css.simulation.resource.scheduler.mapper.ClusterMapper;
import com.css.simulation.resource.scheduler.mapper.ManualProjectMapper;
import com.css.simulation.resource.scheduler.mapper.UserMapper;
import com.css.simulation.resource.scheduler.pojo.po.UserPO;
import com.css.simulation.resource.scheduler.pojo.to.KubernetesNodeTO;
import com.css.simulation.resource.scheduler.pojo.to.PrefixTO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 任务结果打分
 */
@Component
@Slf4j
public class ProjectUtil {

    @Resource
    ManualProjectMapper manualProjectMapper;
    @Resource
    AutoSubProjectMapper autoSubProjectMapper;
    @Resource
    UserMapper userMapper;
    @Resource
    ClusterMapper clusterMapper;
    @Resource
    KafkaTemplate<String, String> kafkaTemplate;
    @Resource
    KubernetesConfiguration kubernetesConfiguration;
    @Resource
    StringRedisTemplate stringRedisTemplate;


    /**
     * 根据并行度获取用于执行的节点列表
     * 根据剩余可用并行度降序排序
     *
     * @return 节点映射（节点名，并行度）
     */
    public Map<String, Integer> getNodeMap(long parallelism) {
        List<KubernetesNodeTO> initialNodeList = kubernetesConfiguration.getNodeList(); // 预设并行度的节点列表
        List<KubernetesNodeTO> restNodeList = new ArrayList<>();    // 剩余并行度的节点列表
        Map<String, Integer> resultNodeMap = new HashMap<>();    // 用于执行的节点映射（节点名，并行度）
        for (KubernetesNodeTO kubernetesNodeTO : initialNodeList) {
            String name = kubernetesNodeTO.getName();
            int maxParallelism = kubernetesNodeTO.getMaxParallelism();
            String restParallelismKey = "node:" + name + ":parallelism";
            String restParallelismString = stringRedisTemplate.opsForValue().get(restParallelismKey);
            int restParallelism;
            if (restParallelismString == null) {    // 如果剩余可用并行度没有值，说明是第一次查询，则重置成最大并行度的预设值
                restParallelism = maxParallelism;
                stringRedisTemplate.opsForValue().set(restParallelismKey, restParallelism + "");
            } else {
                restParallelism = Integer.parseInt(restParallelismString);
                kubernetesNodeTO.setMaxParallelism(restParallelism);
            }
            if (restParallelism > 0) {
                restNodeList.add(kubernetesNodeTO);
                resultNodeMap.put(name, 0);
            }
        }

        for (int i = 0; i < parallelism; i++) {
            // 每次降序排序都取剩余并行度最大的一个。
            restNodeList.sort((o1, o2) -> (int) (o2.getMaxParallelism() - o1.getMaxParallelism()));
            KubernetesNodeTO tempNode = restNodeList.get(0);
            String tempNodeName = tempNode.getName();
            int tempParallelism = tempNode.getMaxParallelism();
            if (tempParallelism > 0) {
                tempNode.setMaxParallelism(tempParallelism - 1);
                resultNodeMap.put(tempNodeName, resultNodeMap.get(tempNodeName) + 1);
            }
        }
        return resultNodeMap;
    }


    public PrefixTO getRedisPrefixByUserIdAndProjectIdAndTaksId(String userId, String projectId, String taskId) {
        //3 获取用户类型（管理员账户、管理员子账户、普通账户、普通子账户）（独占、共享）
        UserPO userPO = userMapper.selectById(userId);
        String roleCode = userPO.getRoleCode();
        String useType = userPO.getUseType();
        String clusterId;
        if (DictConstants.ROLE_CODE_SYSADMIN.equals(roleCode) || DictConstants.ROLE_CODE_ADMIN.equals(roleCode)) {  //3-1 管理员账户和管理员子账户直接执行
            clusterId = DictConstants.SYSTEM_CLUSTER_ID;
        } else if (DictConstants.ROLE_CODE_UESR.equals(roleCode)) { //3-2 普通账户，不管是独占还是共享，都在自己的集群里排队，根据自己的独占节点排队
            clusterId = clusterMapper.selectByUserId(userId).getId();
        } else if (DictConstants.ROLE_CODE_SUBUESR.equals(roleCode)) {
            if (DictConstants.USE_TYPE_EXCLUSIVE.equals(useType)) {   //3-3 普通子账户，根据自己的独占节点排队
                clusterId = clusterMapper.selectByUserId(userId).getId();
            } else {    //3-4 共享子账户，根据父账户的共享节点排队
                String parentUserId = userPO.getCreateUserId();
                clusterId = clusterMapper.selectByUserId(parentUserId).getId();
            }
        } else {
            throw new RuntimeException("ProjectUtil--getRedisPrefixByUserIdAndProjectIdAndTaksId 未知账户类型，无法获取集群信息！");
        }
        String clusterPrefix = "cluster:" + clusterId;
        String clusterRunningPrefix = clusterPrefix + ":running";
        String projectRunningKey = clusterRunningPrefix + ":" + projectId;
        String taskTickKey = projectRunningKey + ":task:" + taskId + ":tick";
        String taskPodKey = projectRunningKey + ":task:" + taskId + ":pod";
        String taskRetryKey = projectRunningKey + ":task:" + taskId + ":retry";
        String taskMessageKey = projectRunningKey + ":task:" + taskId + ":message";

        return PrefixTO.builder()
                .clusterPrefix(clusterPrefix)
                .clusterRunningPrefix(clusterRunningPrefix)
                .projectRunningKey(projectRunningKey)
                .taskTickKey(taskTickKey)
                .taskPodKey(taskPodKey)
                .taskRetryKey(taskRetryKey)
                .taskMessageKey(taskMessageKey)
                .build();

    }


    public PrefixTO getRedisPrefixByUserIdAndProjectId(String userId, String projectId) {
        //3 获取用户类型（管理员账户、管理员子账户、普通账户、普通子账户）（独占、共享）
        UserPO userPO = userMapper.selectById(userId);
        String roleCode = userPO.getRoleCode();
        String useType = userPO.getUseType();
        String clusterId;
        if (DictConstants.ROLE_CODE_SYSADMIN.equals(roleCode) || DictConstants.ROLE_CODE_ADMIN.equals(roleCode)) {  //3-1 管理员账户和管理员子账户直接执行
            clusterId = DictConstants.SYSTEM_CLUSTER_ID;
        } else if (DictConstants.ROLE_CODE_UESR.equals(roleCode)) { //3-2 普通账户，不管是独占还是共享，都在自己的集群里排队，根据自己的独占节点排队
            clusterId = clusterMapper.selectByUserId(userId).getId();
        } else if (DictConstants.ROLE_CODE_SUBUESR.equals(roleCode)) {
            if (DictConstants.USE_TYPE_EXCLUSIVE.equals(useType)) {   //3-3 普通子账户，根据自己的独占节点排队
                clusterId = clusterMapper.selectByUserId(userId).getId();
            } else {    //3-4 共享子账户，根据父账户的共享节点排队
                String parentUserId = userPO.getCreateUserId();
                clusterId = clusterMapper.selectByUserId(parentUserId).getId();
            }
        } else {
            throw new RuntimeException("ProjectUtil--getRedisPrefixByUserIdAndProjectIdAndTaksId 未知账户类型，无法获取集群信息！");
        }
        String clusterPrefix = "cluster:" + clusterId;
        String clusterRunningPrefix = clusterPrefix + ":running";
        String clusterWaitingPrefix = clusterPrefix + ":waiting";
        String projectRunningKey = clusterRunningPrefix + ":" + projectId;
        String projectWaitingKey = clusterWaitingPrefix + ":" + projectId;
        String projectCheckKey = projectRunningKey + ":check";

        return PrefixTO.builder()
                .clusterPrefix(clusterPrefix)
                .clusterRunningPrefix(clusterRunningPrefix)
                .clusterWaitingPrefix(clusterWaitingPrefix)
                .projectRunningKey(projectRunningKey)
                .projectWaitingKey(projectWaitingKey)
                .projectCheckKey(projectCheckKey)
                .build();
    }


    public PrefixTO getRedisPrefixByClusterIdAndProjectId(String clusterId, String projectId) {
        String clusterPrefix = "cluster:" + clusterId;
        String clusterRunningPrefix = clusterPrefix + ":running";
        String clusterWaitingPrefix = clusterPrefix + ":waiting";
        String projectRunningKey = clusterRunningPrefix + ":" + projectId;
        String projectWaitingKey = clusterWaitingPrefix + ":" + projectId;
        String projectCheckKey = clusterWaitingPrefix + ":" + projectId + ":check";

        return PrefixTO.builder()
                .clusterPrefix(clusterPrefix)
                .clusterRunningPrefix(clusterRunningPrefix)
                .clusterWaitingPrefix(clusterWaitingPrefix)
                .projectRunningKey(projectRunningKey)
                .projectWaitingKey(projectWaitingKey)
                .projectCheckKey(projectCheckKey)
                .build();

    }


    public List<String> getRunningProjectList(Set<String> clusterRunningKeySet) {
        return clusterRunningKeySet.stream().filter(key -> StringUtil.countSubString(key, ":") == 3).collect(Collectors.toList());
    }

    public void sendMessage(String topic, String message) {
        kafkaTemplate.send(topic, message);
    }


    public PrefixTO getRedisPrefixByProjectIdAndProjectType(String projectId, String projectType) {
        String userId;
        if (DictConstants.PROJECT_TYPE_MANUAL.equals(projectType)) {
            userId = manualProjectMapper.selectCreateUserById(projectId);
        } else if (DictConstants.PROJECT_TYPE_AUTO_SUB.equals(projectType)) {
            userId = autoSubProjectMapper.selectCreateUserById(projectId);
        } else {
            throw new RuntimeException("PrjectUtil--getRedisPrefixByProjectIdAndProjectType 未知的项目类型！");
        }

        //3 获取用户类型（管理员账户、管理员子账户、普通账户、普通子账户）（独占、共享）
        UserPO userPO = userMapper.selectById(userId);
        String roleCode = userPO.getRoleCode();
        String useType = userPO.getUseType();
        String clusterId;
        if (DictConstants.ROLE_CODE_SYSADMIN.equals(roleCode) || DictConstants.ROLE_CODE_ADMIN.equals(roleCode)) {  //3-1 管理员账户和管理员子账户直接执行
            clusterId = DictConstants.SYSTEM_CLUSTER_ID;
        } else if (DictConstants.ROLE_CODE_UESR.equals(roleCode)) { //3-2 普通账户，不管是独占还是共享，都在自己的集群里排队，根据自己的独占节点排队
            clusterId = clusterMapper.selectByUserId(userId).getId();
        } else if (DictConstants.ROLE_CODE_SUBUESR.equals(roleCode)) {
            if (DictConstants.USE_TYPE_EXCLUSIVE.equals(useType)) {   //3-3 普通子账户，根据自己的独占节点排队
                clusterId = clusterMapper.selectByUserId(userId).getId();
            } else {    //3-4 共享子账户，根据父账户的共享节点排队
                String parentUserId = userPO.getCreateUserId();
                clusterId = clusterMapper.selectByUserId(parentUserId).getId();
            }
        } else {
            throw new RuntimeException("ProjectUtil--getRedisPrefixByUserIdAndProjectIdAndTaksId 未知账户类型，无法获取集群信息！");
        }
        String clusterPrefix = "cluster:" + clusterId;
        String clusterRunningPrefix = clusterPrefix + ":running";
        String clusterWaitingPrefix = clusterPrefix + ":waiting";
        String projectRunningKey = clusterRunningPrefix + ":" + projectId;
        String projectWaitingKey = clusterWaitingPrefix + ":" + projectId;
        String projectCheckKey = projectRunningKey + ":check";

        return PrefixTO.builder()
                .clusterPrefix(clusterPrefix)
                .clusterRunningPrefix(clusterRunningPrefix)
                .clusterWaitingPrefix(clusterWaitingPrefix)
                .projectRunningKey(projectRunningKey)
                .projectWaitingKey(projectWaitingKey)
                .projectCheckKey(projectCheckKey)
                .build();
    }

}
